Package: gap
Version: 1.2.3-1
Date: 2021-4-21
Title: Genetic Analysis Package
Author: Jing Hua Zhao and colleagues with inputs from Kurt Hornik and
        Brian Ripley
Maintainer: Jing Hua Zhao <jinghuazhao@hotmail.com>
URL: https://github.com/jinghuazhao/R
BugReports: https://github.com/jinghuazhao/R/issues
Depends: R (>= 2.10)
Suggests: BradleyTerry2, MASS, Matrix, MCMCglmm, R2jags, bdsmatrix,
        calibrate, circlize, coda, cowplot, coxme, dplyr, foreign,
        forestplot, gap.datasets, genetics, ggplot2, grid, haplo.stats,
        jsonlite, kinship2, lattice, magic, matrixStats, meta, metafor,
        mets, nlme, pedigree, pedigreemm, plotly, plotrix, reshape,
        rmeta, rms, survival
LazyData: Yes
LazyLoad: Yes
Description: It is designed as an integrated package for genetic data
        analysis of both population and family data. Currently, it
        contains functions for sample size calculations of both
        population-based and family-based designs, probability of
        familial disease aggregation, kinship calculation, statistics
        in linkage analysis, and association analysis involving genetic
        markers including haplotype analysis with or without environmental
        covariates. Over years, the package has been developed in-between
        many projects hence also in line with the name (gap).
License: GPL (>= 2)
NeedsCompilation: yes
Repository: CRAN
Packaged: 2021-04-21 07:07:58 UTC; jhz22
Date/Publication: 2021-04-21 09:40:09 UTC
Built: R 4.0.3; x86_64-w64-mingw32; 2021-04-21 13:50:15 UTC; windows
Archs: x64
