useDynLib(gap, .registration = TRUE)
export(AE3, ab, allele.recode, a2g, asplot,
   BFDP, bt, b2r,
   chr_pos_a1_a2, cis.vs.trans.classification, Cox.est, Cox.T, ccsize, chow.test, circos.cis.vs.trans.plot, circos.cnvplot, circos.mhtplot, cnvplot, comp.score, cov.invlogit, cs,
   DevH0dominant,
   DevH0dominant.est,
   DevH0recessive,
   DevH0recessive.est,
   DevHaGdominant,
   DevHaGdominant.est,
   DevHaGrecessive,
   DevHaGrecessive.est,
   ESplot,
   fbsize, FPRP,
   g2a, g2a.c, gc.control, gc.em, gcode, gcontrol, gcontrol2, gc.lambda, gcp, genecounting, geno.recode, 
        getPTE, getb1star, gif, grec2g, gsmr, MCMCgrm,
   h2, h2G, h2GE, h2l, hap, hap.control, hap.em, hap.score, hmht.control, htr,
        h2.jags, hwe, hwe.cc, hwe.hardy, hwe.jags, HapDesign, HapFreqSE,
   inv_chr_pos_a1_a2, invlogit, invnormal,
   KCC, k, klem, kin.morgan,
   LD22, LDkl, lambda1000, logit, log10p, log10pvalue, logp,
   METAL_forestplot, m2plem, makeRLEplot, makeped, masize, metap, metareg, mhtplot, mhtplot2, mhtplot2d, mhtplot3d, mhtplot.trunc, mht.control,
        mia, miamiplot, micombine, mr.boot, mtdt, mtdt2, muvar, mvmeta,
   PARn, pbsize, pbsize2, pedtodot, pfc, pfc.sim, pgc, plem2m, plot.hap.score, print.hap.score, pvalue,
   qqfun, qqunif,
   read.ms.output, revhap, revhap.i, ReadGRM, ReadGRMBin, ReadGRMPLINK, ReadGRMPCA,
   s2k, se.exp, se.invlogit, sentinels, snp.ES, snp.HWE, snp.PAR, snptest_sample, solve_skol,
   toETDT, tscc,
   ungcode,
   VR,
   weighted.median,whscore, WriteGRM, WriteGRMBin, WriteGRMSAS,
   x2,
   z)
S3method(plot, hap.score)
S3method(print, hap.score)
import(utils)
importFrom("graphics", "abline", "arrows", "axis", "box", "identify",
           "legend", "lines", "mtext", "par", "plot", "points",
           "segments", "text")
importFrom("grDevices", "palette", "xy.coords")
importFrom("stats", "as.formula", "coefficients", "cor", "dnorm",
           "glm", "integrate", "median", "nlm", "pchisq", "pf",
           "pnorm", "ppoints", "pt", "qchisq", "qnorm", "qqplot", "qt",
           "quantile", "rbinom", "rexp", "rnorm", "runif", "var")
importFrom("grDevices", "dev.off")
importFrom("graphics", "title")
importFrom("graphics", "boxplot")
importFrom("stats", "coef", "lm", "sd")

